﻿
using System.Net.Sockets;
using System.Text;
using GatewaySCO;
using Serilog;

public class CHECPrueba
{
    private const string GatewayServer = "127.0.0.1";
    private const int GatewayPort = 6690;

    static string mensaje1 = @"soeps~Message-Type=REQ|Session-Id=400|~<?xml version=""1.0"" encoding=""UTF-8""?><scsns:Initialize xmlns:scsns=""http://bc.si.retail.ibm.com/POSBCSchema"" xmlns:xsi=""http://www.w3.org/2001/XMLSchema-instance"" xsi:schemaLocation=""http://bc.si.retail.ibm.com/POSBCSchema C:\\PosBc\\POSBCSchema_main.xsd"" >
         <InitializeRequest>
            <OperatorID>NO_DEFAULT</OperatorID
            <TerminalNumber>400</TerminalNumber>
            <Recovery>false</Recovery>
         </InitializeRequest>
      </scsns:Initialize>";

    static string mensaje2 = @"soeps~Message-Type=REQ|Session-Id=400|~<?xml version=""1.0"" encoding=""UTF-8""?><scsns:QueryStatus xmlns:scsns=""http://bc.si.retail.ibm.com/POSBCSchema"" xmlns:xsi=""http://www.w3.org/2001/XMLSchema-instance"" xsi:schemaLocation=""http://bc.si.retail.ibm.com/POSBCSchema C:\\PosBc\\POSBCSchema_main.xsd"" >
      <QueryStatusRequest/>
    </scsns:QueryStatus>";

    static string mensaje3 = @"soeps~Message-Type=REQ|Session-Id=400|~<?xml version=""1.0"" encoding=""UTF-8""?><scsns:ReportStatusEvents xmlns:scsns=""http://bc.si.retail.ibm.com/POSBCSchema"" xmlns:xsi=""http://www.w3.org/2001/XMLSchema-instance"" xsi:schemaLocation=""http://bc.si.retail.ibm.com/POSBCSchema C:\\PosBc\\POSBCSchema_main.xsd"" >
      <ReportStatusEventsRequest>
      <RequestID>101</RequestID></ReportStatusEventsRequest>
    </scsns:ReportStatusEvents>";

    public static async Task Main(string[] args)
    {
        Log.Logger = new LoggerConfiguration()
            .WriteTo.Console()
            .CreateLogger();

        string[] mensajes = [CHECPrueba.mensaje1, CHECPrueba.mensaje2, CHECPrueba.mensaje3];

        int i = 0;

        try
        {
            using TcpClient client = new TcpClient();
            await client.ConnectAsync(GatewayServer, GatewayPort);
            Log.Information("Conectado al Gateway.");

            using NetworkStream stream = client.GetStream();

            int messageNumber = 1;
            while (true)
            {
                // Remitir un mensaje numerado al Gateway
                string message = mensajes[i++];
                if (i == mensajes.Length - 1) i = 0;

                byte[] messageBytes = Encoding.UTF8.GetBytes(message);
                byte[] lengthBytes = BitConverter.GetBytes(messageBytes.Length);

                if (BitConverter.IsLittleEndian)
                {
                    Array.Reverse(lengthBytes);
                }

                byte[] bytesMsj = Util.ConcatenaArreglosBytes(lengthBytes, messageBytes);

                await stream.WriteAsync(bytesMsj, 0, bytesMsj.Length);
                Log.Information($"Mensaje {messageNumber} enviado al Gateway: {message}");

                // Esperar por una o más respuestas
                await ReceiveResponsesAsync(stream);

                messageNumber++;
                await Task.Delay(8000); // Esperar 5 segundos antes de enviar el siguiente mensaje
            }
        }
        catch (Exception ex)
        {
            Log.Error(ex, "Error en CHECPrueba");
        }
        finally
        {
            Log.CloseAndFlush();
        }
    }

    private static async Task ReceiveResponsesAsync(NetworkStream stream)
    {
        byte[] lengthBuffer = new byte[4];

        while (stream.DataAvailable)
        {
            // Leer longitud del mensaje de respuesta
            int bytesRead = await stream.ReadAsync(lengthBuffer, 0, 4);
            if (bytesRead == 0) break; // La conexión se ha cerrado


            int messageLength = Util.LongitudCodificada(lengthBuffer);
            Log.Information("Mensajde Gateway longitud {longitud}", messageLength);
            byte[] messageBuffer = new byte[messageLength];
            bytesRead = await stream.ReadAsync(messageBuffer, 0, messageLength);
            if (bytesRead == 0) continue; // La conexión se ha cerrado

            string response = Encoding.UTF8.GetString(messageBuffer, 0, bytesRead);
            Log.Information($"Respuesta recibida: {response}");
        }
    }
}

