﻿using System.Net;
using System.Net.Sockets;
using System.Text;
using GatewaySCO;
using Serilog;

public class PosbcServer
{
    private const int Port = 6697;
    private static int messageCounter = 1;

    static string mensaje1 = @"soeps~Session-Id=400|Message-Type=EVENT|~<?xml version=""1.0"" encoding=""UTF-8""?>
<schema:POSBCStatusEvent xmlns:schema=""http://bc.si.retail.ibm.com/POSBCSchema"">
  <POSStatus>
    <Severity>INFO</Severity>
    <Status>CONNECTING_TO_POS</Status>
    <StatusMessage>Connecting to POS system</StatusMessage>
  </POSStatus>
</schema:POSBCStatusEvent>";

    static string mensajeTerminar = @"soeps~Session-Id=400|Message-Type=RESP|~<?xml version=""1.0"" encoding=""UTF-8""?>
<schema:TerminateResponse xmlns:schema=""http://bc.si.retail.ibm.com/POSBCSchema"">
<TerminateResult>
<RequestID>110</RequestID>
</TerminateResult>
</schema:TerminateResponse>";


    public static async Task Main()
    {
        Log.Logger = new LoggerConfiguration()
            .WriteTo.Console()
            .CreateLogger();

        try
        {
            TcpListener listener = new TcpListener(IPAddress.Any, Port);
            listener.Start();
            Log.Information($"POSBC escuchando en el puerto {Port}...");

            while (true)
            {
                TcpClient gatewayClient = await listener.AcceptTcpClientAsync();
                Log.Information("Gateway conectado.");
                _ = Task.Run(() => HandleGatewayClientAsync(gatewayClient));
            }
        }
        catch (Exception ex)
        {
            Log.Error(ex, "Error en el servidor POSBC");
        }
        finally
        {
            Log.CloseAndFlush();
        }
    }

    private static async Task HandleGatewayClientAsync(TcpClient gatewayClient)
    {
        using (gatewayClient)
        using (NetworkStream stream = gatewayClient.GetStream())
        {
            byte[] buffer = new byte[4096];

            while (true)
            {
                // Leer longitud del mensaje desde Gateway
                byte[] lengthBuffer = new byte[4];
                int bytesRead = await stream.ReadAsync(lengthBuffer, 0, 4);
                if (bytesRead == 0) continue; // Si no se lee nada, continuar esperando

                int messageLength = Util.LongitudCodificada(lengthBuffer);
                Log.Information("Mensaje de Gateway longitud {longitud}", messageLength);
                byte[] messageBuffer = new byte[messageLength];
                bytesRead = await stream.ReadAsync(messageBuffer, 0, messageLength);
                if (bytesRead == 0) continue; // Si no se lee nada, continuar esperando

                string message = Encoding.UTF8.GetString(messageBuffer, 0, bytesRead);
                Log.Information($"Mensaje recibido del Gateway: {message}");

                // Esperar 2 segundos antes de enviar la primera respuesta
                await Task.Delay(1000);
                //await EnviarMensaje(stream, $"Respuesta {messageCounter++} desde POSBC");
                await EnviarMensaje(stream, mensaje1);

                // Esperar 2 segundos y enviar dos mensajes adicionales
                await Task.Delay(1000);
                //await EnviarMensaje(stream, $"Mensaje adicional {messageCounter++} desde POSBC");
                await EnviarMensaje(stream, mensaje1);
                await Task.Delay(1000);
                //await EnviarMensaje(stream, $"Mensaje adicional {messageCounter++} desde POSBC");
                await EnviarMensaje(stream, mensajeTerminar);
                await Task.Delay(1000);
                break;
            }
        }
    }

    private static async Task EnviarMensaje(NetworkStream stream, string mensaje)
    {
        byte[] messageBytes = Encoding.UTF8.GetBytes(mensaje);
        byte[] lengthBytes = BitConverter.GetBytes(messageBytes.Length);

        if (BitConverter.IsLittleEndian)
        {
            Array.Reverse(lengthBytes);
        }
        byte[] bytesMsj = Util.ConcatenaArreglosBytes(lengthBytes, messageBytes);
        // await stream.WriteAsync(lengthBytes, 0, 4);
        // await stream.WriteAsync(messageBytes, 0, messageBytes.Length);
        await stream.WriteAsync(bytesMsj);

        Log.Information($"Mensaje enviado al Gateway: {mensaje}");
    }
}
