﻿using Serilog;
using Serilog.Core;
using Serilog.Events;
using Microsoft.Extensions.Configuration;
using EvaPosSCOSrv;
using EvaPosSrvAplicacionImp;
using EvaPosSrvRespImp;
using gatewaySCO.POSBC;
using GatewayCHEC.Servidor;
using Servidor.TcpGateway;

namespace GatewaySCO
{

    /// <summary>
    /// EvaPos-API : servidor api, sockets y rest.
    /// Usa Serilog para bitácora.
    /// </summary> 
    public class ProgramGatewaySCO
    {
        public static async Task Main(string[] args)
        {
            Config(args);
            TcpGateway gateway = null;
            switch (Entorno<Config>.Instancia.Get().POS)
            {
                case "POSBC":
                    gateway = new(
                        Entorno<Config>.Instancia.Get().PortGateway,
                        Entorno<ConfigPOSBC>.Instancia.Get().IpPOSBC,
                        Entorno<ConfigPOSBC>.Instancia.Get().PortPOSBC);
                    break;
                case "POSBCTest":
                    gateway = new(
                        Entorno<Config>.Instancia.Get().PortGateway,
                        Entorno<ConfigPOSBCTest>.Instancia.Get().IpPOSBC,
                        Entorno<ConfigPOSBCTest>.Instancia.Get().PortPOSBC);
                    break;
                default:
                    throw new Exception("Tipo de POS en sección Config de archivo de configuración no reconocido.");
            }
            await gateway.Start();
        }


        /// <summary>
        /// Lee configuración de archivo appsettings.json
        /// La configuración se almacen en clase Entorno para consulta desde
        /// cualquier parte del programa.
        /// </summary>
        /// <param name="args"></param>
        /// <exception cref="ApplicationException"></exception>
        public static void Config(string[] args)
        {
            // TODO - opción de incluir la activación en la cadena de configuración.

            // Instancia objeto de configuración, usa proveedor json y variables ambiente.
            IConfigurationRoot configBuilder = new ConfigurationBuilder()
                .AddJsonFile("appsettings.json")
                .AddEnvironmentVariables()
                .Build();

            // TODO Sacar la ruta de logs del objeto de configuración adecuadamente.
            string? rutaLogs = configBuilder["Serilog:WriteTo:1:Args:path"];
            Console.WriteLine($"Archivo de logs en {rutaLogs}");

            // Instancia Serilog según parámetros en archivo de configuración.
            Log.Logger = new LoggerConfiguration()
                .ReadFrom.Configuration(configBuilder)
                .CreateLogger();

            // Lee del archivo de configuración la configuración general.
            Config config = configBuilder.GetRequiredSection("GatewayConfig").Get<Config>()
                ?? throw new ApplicationException("Archivo de configuración sin sección 'GatewayConfig'.");
            Log.Information(config.ToString());
            Entorno<Config>.Instancia.Set(config);

            if (config.POS == "POSBC")
            {
                ConfigPOSBC configPOSBC = configBuilder.GetRequiredSection("POSBC").Get<ConfigPOSBC>()
                    ?? throw new ApplicationException("Archivo de configuración sin sección 'POSBC'.");
                Log.Information(configPOSBC.ToString());
                Entorno<ConfigPOSBC>.Instancia.Set(configPOSBC);
                Entorno<ConfigPOSBC>.Instancia.Get().IpPOSBC = configPOSBC.IpPOSBC;
                Entorno<ConfigPOSBC>.Instancia.Get().PortPOSBC = configPOSBC.PortPOSBC;
            }

            if (config.POS == "POSBCTest")
            {
                ConfigPOSBCTest configPOSBCTest = configBuilder.GetRequiredSection("POSBCTest").Get<ConfigPOSBCTest>()
                    ?? throw new ApplicationException("Archivo de configuración sin sección 'POSBCTest'.");
                Log.Information(configPOSBCTest.ToString());
                Entorno<ConfigPOSBCTest>.Instancia.Set(configPOSBCTest);
                Entorno<ConfigPOSBCTest>.Instancia.Get().IpPOSBC = configPOSBCTest.IpPOSBC;
                Entorno<ConfigPOSBCTest>.Instancia.Get().PortPOSBC = configPOSBCTest.PortPOSBC;
            }
        }
    }
}
